/*
 * Copyright (c) 2006-2014 Rogério Liesenfeld
 * This file is subject to the terms of the MIT license (see LICENSE.txt).
 */
package mockit.internal.mockups;

import java.lang.reflect.*;

import static java.lang.reflect.Modifier.isNative;

import org.jetbrains.annotations.*;

import mockit.*;
import mockit.internal.*;
import mockit.internal.state.*;
import mockit.internal.util.*;

public final class MockMethodBridge extends MockingBridge
{
   public static final MockingBridge MB = new MockMethodBridge();

   private MockMethodBridge() { super(MockMethodBridge.class); }

   @Override @Nullable
   public Object invoke(@Nullable Object mocked, Method method, @NotNull Object[] args) throws Throwable
   {
      String mockedClassDesc = (String) args[1];
      int mockedAccess = (Integer) args[2];
      String mockDesc = (String) args[4];

      if (notToBeMocked(mocked, mockedClassDesc)) {
         return isNative(mockedAccess) ? DefaultValues.computeForReturnType(mockDesc) : Void.class;
      }

      String mockClassDesc = (String) args[0];
      String mockName = (String) args[3];
      int mockStateIndex = (Integer) args[5];
      Integer mockInstanceIndex = (Integer) args[6];
      Object[] mockArgs = extractMockArguments(7, args);

      return callMock(
         mocked, mockClassDesc, mockedClassDesc, mockName, mockDesc, mockStateIndex, mockInstanceIndex, mockArgs);
   }

   @Nullable
   private static Object callMock(
      @Nullable Object mocked, @NotNull String mockClassDesc, @NotNull String mockedClassDesc,
      @NotNull String mockOrMockedName, @NotNull String mockOrMockedDesc, int mockStateIndex,
      @Nullable Integer mockInstanceIndex, @NotNull Object[] mockArgs)
      throws Throwable
   {
      Object mock;
      Class<?> mockClass;

      if (mockInstanceIndex != null) {
         mock = TestRun.getMock(mockInstanceIndex);
         mockClass = mock.getClass();
      }
      else {
         mock = null;
         String mockClassName = mockClassDesc.replace('/', '.');
         mockClass = ClassLoad.loadClass(mockClassName);
      }

      MockState mockState =
         mockStateIndex < 0 ? null : TestRun.getMockStates().getMockState(mockClassDesc, mockStateIndex);
      Class<?>[] paramClasses;
      Object result;

      if (mockState == null) {
         paramClasses = TypeDescriptor.getParameterTypes(mockOrMockedDesc);
         result = MethodReflection.invokeWithCheckedThrows(mockClass, mock, mockOrMockedName, paramClasses, mockArgs);
         return result;
      }
      else {
         Method mockMethod;

         if (mockState.mockMethod.hasInvocationParameter) {
            MockInvocation invocation;
            Object[] executionArgs;

            if (mockState.shouldProceedIntoRealImplementation(mocked, mockedClassDesc)) {
               return Void.class;
            }
            else if (mockState.mockMethod.isAdvice) {
               paramClasses = new Class[] {Invocation.class};
               mockMethod = mockState.getMockMethod(mockClass, paramClasses);

               invocation =
                  new MockInvocation(mocked, mockArgs, mockState, mockedClassDesc, mockOrMockedName, mockOrMockedDesc);
               executionArgs = new Object[] {invocation};
            }
            else {
               paramClasses = TypeDescriptor.getParameterTypes(mockOrMockedDesc);
               mockMethod = mockState.getMockMethod(mockClass, paramClasses);

               String mockedDesc = mockState.mockMethod.mockDescWithoutInvocationParameter;
               invocation =
                  new MockInvocation(mocked, mockArgs, mockState, mockedClassDesc, mockOrMockedName, mockedDesc);
               executionArgs = ParameterReflection.argumentsWithExtraFirstValue(mockArgs, invocation);
            }

            result = MethodReflection.invokeWithCheckedThrows(mock, mockMethod, executionArgs);
            return invocation.shouldProceedIntoConstructor() ? Void.class : result;
         }
         else {
            paramClasses = TypeDescriptor.getParameterTypes(mockOrMockedDesc);
            mockMethod = mockState.getMockMethod(mockClass, paramClasses);

            result = MethodReflection.invokeWithCheckedThrows(mock, mockMethod, mockArgs);
            return result;
         }
      }
   }
}
